use std::{fs::File, path::Path};

use eyre::Context;
use log::debug;
use rayon::iter::ParallelIterator;

use crate::{
    chunk::{read_chunks_messages, MessageData},
    index::BagIndex,
    info::BagInfo,
    message::compute_layout,
    reader::MmapReader,
    Result,
};

pub struct Bag {
    reader: MmapReader,
    index: BagIndex,
}

impl Bag {
    pub fn open<P: AsRef<Path>>(path: P) -> Result<Self> {
        let bag_file = File::open(path).wrap_err("could not open bag file")?;
        let mut reader = MmapReader::map(bag_file).wrap_err("could not map bag file memory")?;

        // TODO: support unindexed bags
        let index = BagIndex::read(&mut reader).wrap_err("error reading bag index")?;
        Ok(Bag { reader, index })
    }

    pub fn compute_message_layouts(&mut self) -> Result<()> {
        for conn in &self.index.connections {
            let layout = compute_layout(conn)?;
            debug!("message layout: {:#?}", layout);
        }
        Ok(())
    }

    pub fn compute_info(&mut self) -> Result<BagInfo> {
        let reader = self.reader.clone();
        BagInfo::compute(|| reader.clone(), &self.index)
    }

    pub fn read_messages(&mut self) -> impl ParallelIterator<Item = Result<MessageData>> + '_ {
        let reader = self.reader.clone();
        read_chunks_messages(move || reader.clone(), &self.index.chunks)
    }
}
